# -*- coding: utf-8 -*-
#   Copyright (C) 2006-2010, 2013-2015 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Handles gdb-like subcommand processing."""


class Subcmd:
    """Gdb-like subcommand handling """
    def __init__(self, name, cmd_obj):
        self.name    = name
        self.cmd_obj = cmd_obj
        self.subcmds = {}
        self.cmdlist = []
        return

    def lookup(self, subcmd_prefix):
        """Find subcmd in self.subcmds"""
        for subcmd_name in list(self.subcmds.keys()):
            if subcmd_name.startswith(subcmd_prefix) \
               and len(subcmd_prefix) >= \
               self.subcmds[subcmd_name].__class__.min_abbrev:
                return self.subcmds[subcmd_name]
            pass
        return None

    def short_help(self, subcmd_cb, subcmd_name, label=False):
        """Show short help for a subcommand."""
        entry = self.lookup(subcmd_name)
        if entry:
            if label:
                prefix = entry.name
            else:
                prefix = ''
                pass
            if hasattr(entry, 'short_help'):
                if prefix: prefix += ' -- '
                self.cmd_obj.msg(prefix + entry.short_help)
                pass
            pass
        else:
            self.undefined_subcmd("help", subcmd_name)
            pass
        return

    def add(self, subcmd_cb):
        """Add subcmd to the available subcommands for this object.
        It will have the supplied docstring, and subcmd_cb will be called
        when we want to run the command. min_len is the minimum length
        allowed to abbreviate the command. in_list indicates with the
        show command will be run when giving a list of all sub commands
        of this object. Some commands have long output like "show commands"
        so we might not want to show that.
        """
        subcmd_name = subcmd_cb.name
        self.subcmds[subcmd_name] = subcmd_cb

        # We keep a list of subcommands to assist command completion
        self.cmdlist.append(subcmd_name)

    def run(self, subcmd_name, arg):
        """Run subcmd_name with args using obj for the environent"""
        entry=self.lookup(subcmd_name)
        if entry:
            entry['callback'](arg)
        else:
            self.cmdproc.undefined_cmd(entry.__class__.name, subcmd_name)
            pass
        return

    # Note: format of help is compatible with ddd.
    def help(self, *args):
        """help for subcommands."""

        print(args)
        subcmd_prefix = args[0]
        if not subcmd_prefix or len(subcmd_prefix) == 0:
            self.msg(self.doc)
            self.msg("""
List of %s subcommands:
""" % (self.name))
            for subcmd_name in self.list():
                self._subcmd_helper(subcmd_name, self, True, True)
            return

        entry = self.lookup(subcmd_prefix)
        if entry and hasattr(entry, 'help'):
            entry.help(args)
        else:
            self.cmd_obj.errmsg("Unknown 'help %s' subcommand %s"
                                % (self.name, subcmd_prefix))

    def list(self):
        l = list(self.subcmds.keys())
        l.sort()
        return l

    def undefined_subcmd(self, cmd, subcmd):
        """Error message when a subcommand doesn't exist"""
        self.cmd_obj.errmsg('Undefined "%s" command: "%s". Try "help".' %
                         (cmd, subcmd,))
        return
    pass

# When invoked as main program, invoke the debugger on a script
if __name__ == '__main__':

    from trepan.processor.command import mock as Mmock
    from trepan.processor.command import base_cmd as Mbase_cmd

    class TestCommand(Mbase_cmd.DebuggerCommand):
        '''Doc string for testing'''

        category = 'data'
        min_args = 0
        max_args = 5
        name = 'test'

        def __init__(self):
            self.name  = 'test'
            return

        def run(self, args): print('test command run')

    class TestTestingSubcommand:
        '''Doc string for test testing subcommand'''

        def __init__(self):
            self.name  = 'testing'
            return

        short_help = 'This is short help for test testing'
        min_abbrev = 4
        in_list    = True

        def run(self, args): print('test testing run')
        pass

    d = Mmock.MockDebugger()
    testcmd    = TestCommand()
    testcmd.debugger = d
    testcmd.proc     = d.core.processor
    testcmdMgr = Subcmd('test', testcmd)
    testsub = TestTestingSubcommand()
    testcmdMgr.add(testsub)

    for prefix in ['tes', 'test', 'testing', 'testing1']:
        x = testcmdMgr.lookup(prefix)
        if x: print(x.name)
        else: print('None')
        pass

    testcmdMgr.short_help(testcmd, 'testing')
    testcmdMgr.short_help(testcmd, 'test', True)
    testcmdMgr.short_help(testcmd, 'tes')
    print(testcmdMgr.list())
    testsub2 = TestTestingSubcommand()
    testsub2.name = 'foobar'
    testcmdMgr.add(testsub2)
    print(testcmdMgr.list())
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2015, 2018 Rocky Bernstein
#

from pygments.styles import STYLE_MAP

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.lib import complete as Mcomplete


style_names = sorted(list(STYLE_MAP.keys()))
def complete(self, prefix):
    return Mcomplete.complete_token(style_names)

class ShowStyle(Mbase_subcmd.DebuggerSubcommand):
    """**show style* *name*

Show the pygments style used in formatting 256-color terminal text.

See also:
---------

`set style`, `show highlight`
"""

    in_list    = True
    min_abbrev = len('sty')
    short_help = 'Set the pygments style'

    def run(self, args):
        if len(args) != 0:
            self.errmsg("Expecting no args")
            return

        style = self.debugger.settings[self.name]
        if style:
            self.msg("Pygments style is %s" % style)
        else:
            self.msg("Pygments style not set")
        return
    pass

if __name__ == '__main__':
    from trepan.processor.command.set_subcmd import __demo_helper__ as Mhelper
    sub = Mhelper.demo_run(ShowStyle)
    d = sub.proc.debugger
    sub.run([])
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013, 2015-2016, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pprint
from columnize import columnize


def pp(val, display_width, msg_nocr, msg, prefix=None):
    if prefix is not None:
        val_len = len(repr(val))
        if val_len + len(prefix) < display_width - 1:
            msg(prefix + " " + repr(val))
            return
        else:
            msg(prefix)
        pass
    if not pprint_simple_array(val, display_width, msg_nocr, msg, "  "):
        print("Can't print_simple_array")
        msg("  " + pprint.pformat(val))
        pass
    return


# Actually... code like this should go in pformat.
# Possibly some will go into columnize.
def pprint_simple_array(val, displaywidth, msg_nocr, msg, lineprefix=""):
    """Try to pretty print a simple case where a list is not nested.
    Return True if we can do it and False if not. """

    if type(val) != list:
        return False

    numeric = True
    for i in range(len(val)):
        if not (type(val[i]) in [bool, float, int]):
            numeric = False
            if not (type(val[i]) in [bool, float, int, bytes]):
                return False
            pass
        pass
    mess = columnize(
        [repr(v) for v in val],
        opts={
            "arrange_array": True,
            "lineprefix": lineprefix,
            "displaywidth": int(displaywidth) - 3,
            "ljust": not numeric,
        },
    )
    msg_nocr(mess)
    return True


if __name__ == "__main__":

    def msg_nocr(m):
        sys.stdout.write(m)
        return

    import sys

    def msg(m):
        print(m)

    pprint_simple_array(range(50), 50, msg_nocr, msg)
    pp([i for i in range(10)], 50, msg_nocr, msg)
    pp(locals(), 50, msg_nocr, msg)
    x = [i for i in range(10)]
    pp(x, 50, msg_nocr, msg, "x = ")
    pp(x, 20, msg_nocr, msg, "x = ")
    pp(x, 32, msg_nocr, msg, "x = ")
    x = [i for i in range(30)]
    l = locals().keys()
    for k in sorted(l):
        pp(eval(k), 80, msg_nocr, msg, prefix="%s =" % k)
        pass
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2015 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.lib import complete as Mcomplete


class InfoBuiltins(Mbase_subcmd.DebuggerSubcommand):
    """**info builtins**

Show the builtin-functions for the current stack frame."""

    max_args   = 1
    min_abbrev = 2
    need_stack = True
    short_help = "Show the builtins for current stack frame"

    def complete(self, prefix):
        completions = sorted(['*'] +
                              self.proc.curframe.f_builtins.keys())
        return Mcomplete.complete_token(completions, prefix)

    def run(self, args):
        if not self.proc.curframe:
            self.errmsg("No frame selected.")
            return False
        names = list(self.proc.curframe.f_builtins.keys())
        if len(args) > 0 and args[0] == '*' :
            self.section("builtins")
            self.msg(self.columnize_commands(names))
        elif len(args) == 0:
            if len(names) > 0:
                self.section("builtins")
                self.msg(self.columnize_commands(names))
        return False
    pass

if __name__ == '__main__':
    from trepan.processor.command import mock, info as Minfo
    from trepan import debugger as Mdebugger
    d = Mdebugger.Debugger()
    d, cp = mock.dbg_setup(d)
    i = Minfo.InfoCommand(cp)
    sub = InfoBuiltins(i)
    import inspect
    cp.curframe = inspect.currentframe()
    sub.run([])
    pass



#!/usr/bin/env python3
'Unit test for trepan.processor.command.disassemble'
import inspect, unittest

from trepan.processor.command import disassemble as Mdis

from cmdhelper import dbg_setup


class TestDisassemble(unittest.TestCase):

    # FIXME: put in a more common place
    # Possibly fix up Mock to include this
    def setup_io(self, command):
        self.clear_output()
        command.msg = self.msg
        command.errmsg = self.errmsg
        command.msg_nocr = self.msg_nocr
        return

    def clear_output(self):
        self.msgs = []
        self.errmsgs = []
        self.last_was_newline = True
        return

    def msg(self, msg):
        self.msg_nocr(msg)
        self.last_was_newline = True
        return

    def msg_nocr(self, msg):
        if self.last_was_newline:
            self.msgs.append('')
            pass
        self.msgs[-1] += msg
        self.last_was_newline = len(msg) == 0
        return

    def errmsg(self, msg):
        self.errmsgs.append(msg)
        pass

    def test_disassemble(self):
        """Test processor.command.disassemble.run()"""

        print("Skipping until disassembly revamp complete")
        assert True
        return
        d, cp = dbg_setup()
        command = Mdis.DisassembleCommand(cp)

        self.setup_io(command)
        command.run(['disassemble'])
        self.assertTrue(len(self.errmsgs) > 0)
        self.assertEqual(len(self.msgs), 0)
        me = self.test_disassemble  # NOQA
        cp.curframe = inspect.currentframe()
        # All of these should work
        for args in (['disassemble'],
                     ['disassemble', 'cp.errmsg'],
                     ['disassemble', 'unittest'],
                     ['disassemble', '1'],
                     ['disassemble', '10', '100'],
                     ['disassemble', '*10', '*30'],
                     ['disassemble', '+', '1'],
                     ['disassemble', '-', '1'],
                     ['disassemble', '1', '2'],
                     ['disassemble', 'me']):
            self.clear_output()
            try:
                command.run(args)
            except NotImplementedError:
                return

            self.assertTrue(len(self.msgs) > 0,
                            "msgs for: %s" % ' '.join(args))
            self.assertEqual(len(self.errmsgs), 0,
                             "errmsgs for: %s %s" % (' '.join(args),
                                                     "\n".join(self.errmsgs)))
            pass
        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
'Unit test for trepan.processor.command.help'
import unittest

from trepan.processor.command import help as Mhelp, mock as Mmock
from trepan.processor import cmdproc as Mcmdproc


class TestHelp(unittest.TestCase):
    """Tests HelpCommand class"""

    def setUp(self):
        self.errors             = []
        self.msgs               = []
        self.d                  = Mmock.MockDebugger()
        self.cp                 = Mcmdproc.CommandProcessor(self.d.core)
        self.cp.intf[-1].msg    = self.msg
        self.cp.intf[-1].errmsg = self.errmsg
        self.cmd                = Mhelp.HelpCommand(self.cp)
        self.cmd.msg            = self.msg
        self.cmd.errmsg         = self.errmsg
        return

    def errmsg(self, msg):
        self.errors.append(msg)
        return

    def msg(self, msg):
        self.msgs.append(msg)
        return

    def test_help_command(self):
        """Test we can run 'help *cmd* for each command"""

        for name in self.cp.commands.keys():
            self.cmd.run(['help', name])
            pass
        self.assertTrue(len(self.msgs) > 0, 'Should get help output')
        self.assertEqual(0, len(self.errors), 'Should not get errors')
        return

    def test_help_categories(self):
        """Test we can run 'help *cmd* for each category"""

        for name in Mhelp.categories.keys():
            self.cmd.run(['help', name])
            pass
        self.assertTrue(len(self.msgs) > 0, 'Should get help output')
        self.assertEqual(0, len(self.errors), 'Should not get errors')

        for name in Mhelp.categories.keys():
            self.cmd.run(['help', name, '*'])
            pass
        self.assertTrue(len(self.msgs) > 0, 'Should get help output')
        self.assertEqual(0, len(self.errors), 'Should not get errors')

        return

    def test_short_help(self):
        """Test each command has some sort of short help"""
        for cmd in list(self.cp.commands.values()):
            self.assertEqual(str, type(cmd.short_help))
            pass
        return

    pass
if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2017, 2020 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inspect
from xdis import findlinestarts

# Our local modules
from trepan.processor.command.base_subcmd import DebuggerSubcommand
from trepan.lib.disassemble import disassemble_bytes
from trepan.misc import wrapped_lines


# FIXME: this could be combined with trepan-xpy's `info pc`, but that
# requires a running program whereas where we just use f_lasti.
# What we have here is more flexible in the presence of exceptions.
# trepan3k would have to be looked over to see if it too would
# work.
class InfoPC(DebuggerSubcommand):
    """**info pc**

List the current program counter or bytecode offset,
and disassemble the instructions around that.

See also:
---------

`info line`, `info program`
"""

    min_abbrev = 2  # Need at least info 'pc'
    max_args = 0
    need_stack = True
    short_help = "Show Program Counter or Instruction Offset information"

    def run(self, args):
        """Program counter."""
        mainfile = self.core.filename(None)
        if self.core.is_running():
            curframe = self.proc.curframe
            if curframe:
                line_no = inspect.getlineno(curframe)
                offset = curframe.f_lasti
                self.msg("PC offset is %d." % offset)
                offset = max(offset, 0)
                code = curframe.f_code
                co_code = code.co_code
                disassemble_bytes(
                    self.msg,
                    self.msg_nocr,
                    co_code,
                    offset,
                    line_no,
                    line_no - 1,
                    line_no + 1,
                    constants=code.co_consts,
                    cells=code.co_cellvars,
                    varnames=code.co_varnames,
                    freevars=code.co_freevars,
                    linestarts=dict(findlinestarts(code)),
                    end_offset=offset + 10,
                )
                pass
            pass
        else:
            if mainfile:
                part1 = "Python program '%s'" % mainfile
                msg = "is not currently running. "
                self.msg(wrapped_lines(part1, msg, self.settings["width"]))
            else:
                self.msg("No Python program is currently running.")
                pass
            self.msg(self.core.execution_status)
            pass
        return False

    pass


if __name__ == "__main__":
    from trepan.processor.command import mock, info as Minfo

    d, cp = mock.dbg_setup()
    i = Minfo.InfoCommand(cp)
    sub = InfoPC(i)
    sub.run([])



# -*- coding: utf-8 -*-
#   Copyright (C) 2009-2010, 2013-2015 Rocky Bernstein
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" Not a command. A stub class used by a command in its 'main' for
demonstrating how the command works."""

import trepan.lib
import trepan.inout
import trepan.interfaces
import trepan.processor.command

import os, sys
from trepan.lib import breakpoint, default


class MockIO:
    def readline(self, prompt="", add_to_history=False):
        print(prompt)
        return "quit"

    def output(self):
        print

    pass


class MockUserInterface:
    def __init__(self):
        self.io = MockIO()
        self.output = MockIO()
        self.debugger_name = "trepan3k"
        return

    def confirm(self, msg, default):
        print("** %s" % msg)
        # Ignore the default.
        return True

    def errmsg(self, msg):
        print("** %s" % msg)
        return

    def finalize(self, last_wishes=None):
        return

    def msg(self, msg):
        print(msg)
        return

    def msg_nocr(self, msg):
        sys.stdout.write(msg)
        return

    pass


class MockProcessor:
    def __init__(self, core):
        self.core = core
        self.debugger = core.debugger
        self.continue_running = False
        self.curframe = None
        self.event2short = {}
        self.frame = None
        self.intf = core.debugger.intf
        self.last_command = None
        self.stack = []
        return

    def get_int(self, arg, min_value=0, default=1, cmdname=None, at_most=None):
        return None

    def undefined_cmd(self, cmd):
        self.intf[-1].errmsg('Undefined mock command: "%s' % cmd)
        return

    pass


# External Egg packages
import tracefilter


class MockDebuggerCore:
    def __init__(self, debugger):
        self.debugger = debugger
        self.execution_status = "Pre-execution"
        self.filename_cache = {}
        self.ignore_filter = tracefilter.TraceFilter([])
        self.bpmgr = breakpoint.BreakpointManager()
        self.processor = MockProcessor(self)
        self.step_ignore = -1
        self.stop_frame = None
        self.last_lineno = None
        self.last_offset = None
        self.last_filename = None
        self.different_line = None
        self.from_ipython = False
        return

    def set_next(self, frame, step_events=None):
        pass

    def stop(self):
        pass

    def canonic(self, filename):
        return filename

    def canonic_filename(self, frame):
        return frame.f_code.co_filename

    def filename(self, name):
        return name

    def is_running(self):
        return "Running" == self.execution_status

    def get_file_breaks(self, filename):
        return []

    pass


class MockDebugger:
    def __init__(self):
        self.intf = [MockUserInterface()]
        self.core = MockDebuggerCore(self)
        self.settings = default.DEBUGGER_SETTINGS
        self.orig_sys_argv = None
        self.program_sys_argv = []
        return

    def stop(self):
        pass

    def restart_argv(self):
        return []

    pass


def dbg_setup(d=None):
    if d is None:
        d = MockDebugger()
    from trepan.processor import cmdproc

    cp = cmdproc.CommandProcessor(d.core)
    return d, cp



# -*- coding: utf-8 -*-
#  Copyright (C) 2009-2010, 2012-2013, 2015 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""A base class for debugger commands.

This file is the one module in this directory that isn't a real command
and commands.py needs to take care to avoid instantiating this class
and storing it as a list of known debugger commands.
"""

NotImplementedMessage = "This method must be overriden in a subclass"

__all__ = ['DebuggerCommand']


class DebuggerCommand:
    """Base Class for Debugger commands. We pull in some helper
    functions for command from module cmdfns."""

    category = 'misc'

    def __init__(self, proc):
        """proc contains the command processor object that this
        command is invoked through.  A debugger field gives access to
        the stack frame and I/O."""
        self.proc = proc

        # Convenience class access. We don't expect that either core
        # or debugger will change over the course of the program
        # execution like errmsg(), msg(), and msg_nocr() might. (See
        # the note below on these latter 3 methods.)
        #
        self.core     = proc.core
        self.debugger = proc.debugger
        self.settings = self.debugger.settings
        return

    aliases = ()
    name    = 'YourCommandName'

    # Note for errmsg, msg, and msg_nocr we don't want to simply make
    # an assignment of method names like self.msg = self.debugger.intf.msg,
    # because we want to allow the interface (intf) to change
    # dynamically. That is, the value of self.debugger may change
    # in the course of the program and if we made such an method assignemnt
    # we wouldn't pick up that change in our self.msg
    def errmsg(self, msg, opts={}):
        """ Convenience short-hand for self.debugger.intf.errmsg """
        try:
            return(self.debugger.intf[-1].errmsg(msg))
        except EOFError:
            # FIXME: what do we do here?
            pass
        return None

    def msg(self, msg, opts={}):
        """ Convenience short-hand for self.debugger.intf.msg """
        try:
            return(self.debugger.intf[-1].msg(msg))
        except EOFError:
            # FIXME: what do we do here?
            pass
        return None

    def msg_nocr(self, msg, opts={}):
        """ Convenience short-hand for self.debugger.intf.msg_nocr """
        try:
            return(self.debugger.intf[-1].msg_nocr(msg))
        except EOFError:
            # FIXME: what do we do here?
            pass
        return None

    def run(self, args):
        """ The method that implements the debugger command.
        Help on the command comes from the docstring of this method.
        """
        raise NotImplementedError(NotImplementedMessage)

    pass

if __name__ == '__main__':
    from trepan.bwprocessor.command import mock
    d, cp = mock.dbg_setup()
    dd = DebuggerCommand(cp)
    dd.msg("hi")
    dd.errmsg("Don't do that")
    pass



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2009-2010, 2013-2015, 2017-2018 Rocky Bernstein
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os

# Our local modules
from trepan.processor.command import base_cmd as Mbase_cmd
from trepan.processor import cmdbreak as Mcmdbreak
from trepan.processor import complete as Mcomplete


class BreakCommand(Mbase_cmd.DebuggerCommand):
    """**break** [*location*] [if *condition*]]

Sets a breakpoint, i.e. stopping point just before the
execution of the instruction specified by *location*.

Without arguments or an empty *location*, the breakpoint is set at the
current stopped location.

See `help syntax location` for detailed information on a location.

If the word `if` is given after *location*, subsequent arguments given
Without arguments or an empty *location*, the breakpoint is set
the current stopped location.

Normally we only allow stopping at lines that we think are
stoppable. If the command has a `!` suffix, force the breakpoint anyway.

Examples:
---------

   break                # Break where we are current stopped at
   break if i < j       # Break at current line if i < j
   break 10             # Break on line 10 of the file we are
                        # currently stopped at
   break! 10            # Break where we are current stopped at, even if
                        # we don't think line 10 is stoppable
   break os.path.join() # Break in function os.path.join
   break x[i].fn()      # break in function specified by x[i].fn
   break x[i].fn() if x # break in function specified by x[i].fn
                        # if x is set
   break os.path:45     # Break on line 45 file holding module os.path
   break myfile.py:2    # Break on line 2 of myfile.py
   break myfile.py:2 if i < j # Same as above but only if i < j
   break "foo's.py":1"  # One way to specify path with a quote
   break 'c:\\foo.bat':1      # One way to specify a Windows file name,
   break '/My Docs/foo.py':1  # One way to specify path with blanks in it

See also:
---------

`info break`, `tbreak`, `condition` and `help syntax location`."""

    aliases       = ('b', 'break!', 'b!')
    category      = 'breakpoints'
    min_args      = 0
    max_args      = None
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = True
    short_help    = 'Set breakpoint at specified line or function'

    complete= Mcomplete.complete_break_linenumber

    def run(self, args):
        force = True if args[0][-1] == '!' else False

        (func, filename, lineno,
         condition) = Mcmdbreak.parse_break_cmd(self.proc, args)
        if not (func == None and filename == None):
            Mcmdbreak.set_break(self, func, filename, lineno, condition,
                                False, args, force=force)
        return

if __name__ == '__main__':
    def doit(cmd, a):
        cmd.current_command = ' '.join(a)
        print(Mcmdbreak.parse_break_cmd(cmd.proc, a))

    import sys
    from trepan import debugger as Mdebugger
    d = Mdebugger.Trepan()
    command = BreakCommand(d.core.processor)
    command.proc.frame = sys._getframe()
    command.proc.setup()

    doit(command, [' '])
    doit(command, ['10'])
    doit(command, [__file__ + ':10'])

    def foo():
        return 'bar'
    doit(command, ['foo'])
    doit(command, ['os.path'])
    doit(command, ['os.path', '5+1'])
    doit(command, ['os.path.join'])
    doit(command, ['if', 'True'])
    doit(command, ['foo', 'if', 'True'])
    doit(command, ['os.path:10', 'if', 'True'])
    command.run(['break'])
    command.run(['break', 'command.run'])
    command.run(['break', '10'])
    command.run(['break', __file__ + ':10'])
    command.run(['break', 'foo'])
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013, 2015 Rocky Bernstein
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
#    02110-1301 USA.
import inspect, os, sys, types

# Our local modules
from trepan.processor.command import base_cmd as Mbase_cmd
from trepan.processor import complete as Mcomplete


class WhatisCommand(Mbase_cmd.DebuggerCommand):
    '''**whatis** *arg*

Prints the information argument which can be a Python expression.

When possible, we give information about:

* type of argument

* doc string for the argument (if a module, class, or function)

* comments around the definition of the argument (module)

* the module it was defined in

* where the argument was defined

We get this most of this information via the *inspect* module.

See also:
--------

`pydocx`, the *inspect* module.'''

    aliases       = ()
    category      = 'data'
    min_args      = 1
    max_args      = None
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = True
    short_help   = 'Print data type of expression EXP'

    complete = Mcomplete.complete_id_and_builtins

    def run(self, args):
        proc = self.proc
        arg = proc.cmd_argstr
        try:
            if not proc.curframe:
                # ?? Should we have set up a dummy globals
                # to have persistence?
                value = eval(arg, None, None)
            else:
                value = eval(arg, proc.curframe.f_globals,
                             proc.curframe.f_locals)
        except:
            t, v = sys.exc_info()[:2]
            if type(t) == str:
                exc_type_name = t
            else: exc_type_name = t.__name__
            if exc_type_name == 'NameError':
                self.errmsg("Name Error: %s" % arg)
            else:
                self.errmsg("%s: %s" % (exc_type_name, proc._saferepr(v)))
            return False

        self.section("What is for %s" % arg)

        get_doc = False
        if inspect.ismethod(value):
            get_doc = True
            self.msg('  method %s%s' %
                     (value.__code__.co_name,
                       inspect.formatargspec(inspect.getargspec(value))))
        elif inspect.isfunction(value):
            get_doc = True
            self.msg('  function %s%s' %
                     (value.__code__.co_name, inspect.signature(value)))
        elif inspect.isabstract(value) or \
             inspect.isbuiltin(value) or \
             inspect.isclass(value) or \
             inspect.isgeneratorfunction(value) or \
             inspect.ismethoddescriptor(value):
            get_doc = True

        self.msg('  type: %s' % type(value))
        doc = inspect.getdoc(value)
        if get_doc and doc:
            self.msg('  doc:\n%s' % doc)
        comments = inspect.getcomments(value)
        if comments:
            self.msg('  comments:\n%s' % comments)
        try:
            m = inspect.getmodule(value)
            if m: self.msg("  module:\t%s" % m)
        except:
            try:
                f = inspect.getfile(value)
                self.msg("  file: %s" % f)
            except:
                pass
            pass
        return False

    pass

if __name__ == '__main__':
    from trepan.processor import cmdproc as Mcmdproc
    from trepan.processor.command import mock as Mmock
    d, cp       = Mmock.dbg_setup()
    command     = WhatisCommand(cp)
    cp.curframe = inspect.currentframe()
    cp.stack, cp.curindex = Mcmdproc.get_stack(cp.curframe, None, None,
                                               cp)

    words = '''5 1+2 thing len trepan os.path.basename WhatisCommand cp
               __name__ Mmock Mbase_cmd.DebuggerCommand'''.split()
    for thing in words:
        cp.cmd_argstr = thing
        command.run(['whatis', thing])
        print('-' * 10)
    pass



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
#   Copyright (C) 2013, 2015, 2020 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
""" The hairy command-line interface to the debugger.
"""
import os, os.path as osp, sys

from optparse import OptionParser

from pyficache import resolve_name_to_path

# Our local modules
from trepan import clifns as Mclifns
from trepan import debugger as Mdebugger, exception as Mexcept, misc as Mmisc
from trepan.lib.file import readable
from trepan.interfaces.bullwinkle import BWInterface

# The name of the debugger we are currently going by.
__title__ = "trepan"

# VERSION.py sets variable VERSION.
from trepan.VERSION import VERSION as __version__


def process_options(debugger_name, pkg_version, sys_argv, option_list=None):
    """Handle debugger options. Set `option_list' if you are writing
    another main program and want to extend the existing set of debugger
    options.

    The options dicionary from opt_parser is return. sys_argv is
    also updated."""
    usage_str = """%prog [debugger-options] [python-script [script-options...]]

       Runs the extended python debugger"""

    # serverChoices = ('TCP','FIFO', None)

    optparser = OptionParser(
        usage=usage_str,
        option_list=option_list,
        version="%%prog version %s" % pkg_version,
    )

    optparser.add_option(
        "-F",
        "--fntrace",
        dest="fntrace",
        action="store_true",
        default=False,
        help="Show functions before executing them. " + "This option also sets --batch",
    )
    optparser.add_option(
        "--basename",
        dest="basename",
        action="store_true",
        default=False,
        help="Filenames strip off basename, " "(e.g. for regression tests)",
    )
    optparser.add_option(
        "--different",
        dest="different",
        action="store_true",
        default=True,
        help="Consecutive stops should have different " "positions",
    )
    optparser.disable_interspersed_args()

    sys.argv = list(sys_argv)
    (opts, sys.argv) = optparser.parse_args()
    dbg_opts = {}

    return opts, dbg_opts, sys.argv


def _postprocess_options(dbg, opts):
    """ Handle options (`opts') that feed into the debugger (`dbg')"""
    # Set dbg.settings['printset']
    print_events = []
    if opts.fntrace:
        print_events = ["c_call", "c_return", "call", "return"]
    # if opts.linetrace: print_events += ['line']
    if len(print_events):
        dbg.settings["printset"] = frozenset(print_events)
        pass

    for setting in (
        "basename",
        "different",
    ):
        dbg.settings[setting] = getattr(opts, setting)
        pass

    dbg.settings["highlight"] = "plain"

    Mdebugger.debugger_obj = dbg
    return


def main(dbg=None, sys_argv=list(sys.argv)):
    """Routine which gets run if we were invoked directly"""
    global __title__

    # Save the original just for use in the restart that works via exec.
    orig_sys_argv = list(sys_argv)
    opts, dbg_opts, sys_argv = process_options(__title__, __version__, sys_argv)
    dbg_opts["orig_sys_argv"] = sys_argv
    dbg_opts["interface"] = BWInterface()
    dbg_opts["processor"] = "bullwinkle"

    if dbg is None:
        dbg = Mdebugger.Trepan(dbg_opts)
        dbg.core.add_ignore(main)
        pass
    _postprocess_options(dbg, opts)

    # process_options has munged sys.argv to remove any options that
    # options that belong to this debugger. The original options to
    # invoke the debugger and script are in global sys_argv
    if len(sys_argv) == 0:
        # No program given to debug. Set to go into a command loop
        # anyway
        mainpyfile = None
    else:
        mainpyfile = sys_argv[0]  # Get script filename.
        if not osp.isfile(mainpyfile):
            mainpyfile = Mclifns.whence_file(mainpyfile)
            is_readable = readable(mainpyfile)
            if is_readable is None:
                print(
                    "%s: Python script file '%s' does not exist"
                    % (__title__, mainpyfile,)
                )
                sys.exit(1)
            elif not is_readable:
                print(
                    "%s: Can't read Python script file '%s'" % (__title__, mainpyfile,)
                )
                sys.exit(1)
                return

        # If mainpyfile is an optimized Python script try to find and
        # use non-optimized alternative.
        mainpyfile_noopt = pyficache.resolve_name_to_file(mainpyfile)
        if mainpyfile != mainpyfile_noopt and readable(mainpyfile_noopt):
            print("%s: Compiled Python script given and we can't use that." % __title__)
            print(
                "%s: Substituting non-compiled name: %s"
                % (__title__, mainpyfile_noopt,)
            )
            mainpyfile = mainpyfile_noopt
            pass

        # Replace trepan's dir with script's dir in front of
        # module search path.
        sys.path[0] = dbg.main_dirname = osp.dirname(mainpyfile)

    # XXX If a signal has been received we continue in the loop, otherwise
    # the loop exits for some reason.
    dbg.sig_received = False

    # if not mainpyfile:
    #     print('For now, you need to specify a Python script name!')
    #     sys.exit(2)
    #     pass

    while True:

        # Run the debugged script over and over again until we get it
        # right.

        try:
            if dbg.program_sys_argv and mainpyfile:
                normal_termination = dbg.run_script(mainpyfile)
                if not normal_termination:
                    break
            else:
                dbg.core.execution_status = "No program"
                dbg.core.processor.process_commands()
                pass

            dbg.core.execution_status = "Terminated"
            dbg.intf[-1].msg("The program finished - quit or restart")
            dbg.core.processor.process_commands()
        except Mexcept.DebuggerQuit:
            break
        except Mexcept.DebuggerRestart:
            dbg.core.execution_status = "Restart requested"
            if dbg.program_sys_argv:
                sys.argv = list(dbg.program_sys_argv)
                part1 = "Restarting %s with arguments:" % dbg.core.filename(mainpyfile)
                args = " ".join(dbg.program_sys_argv[1:])
                dbg.intf[-1].msg(
                    Mmisc.wrapped_lines(part1, args, dbg.settings["width"])
                )
            else:
                break
        except SystemExit:
            # In most cases SystemExit does not warrant a post-mortem session.
            break
        pass

    # Restore old sys.argv
    sys.argv = orig_sys_argv
    return


# When invoked as main program, invoke the debugger on a script
if __name__ == "__main__":
    main()
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path as osp

from trepan.processor.command.base_submgr import SubcommandMgr


class SetCommand(SubcommandMgr):
    """**set** *set subcommand*

Modifies parts of the debugger environment.

You can give unique prefix of the name of a subcommand to get
information about just that subcommand.

Type `set` for a list of *set* subcommands and what they do.
Type `help set *` for just the list of *set* subcommands.
"""

    category = "data"
    min_args = 0
    max_args = None
    name = osp.basename(__file__).split(".")[0]
    need_stack = False
    short_help = "Modify parts of the debugger environment"


if __name__ == "__main__":
    from trepan.processor.command import mock

    d, cp = mock.dbg_setup()
    command = SetCommand(cp, "set")
    command.run(["set"])
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2015 Rocky Bernstein <rocky@gnu.org>
''' Common I/O routines'''

# Note for errmsg, msg, and msg_nocr we don't want to simply make
# an assignment of method names like self.msg = self.debugger.intf.msg,
# because we want to allow the interface (intf) to change
# dynamically. That is, the value of self.debugger may change
# in the course of the program and if we made such an method assignemnt
# we wouldn't pick up that change in our self.msg
def errmsg(proc_obj, message, opts={}):
    response = proc_obj.response
    if 'set_name' in opts: response['name'] = 'error'
    return response['errs'].append(message)


def msg(proc_obj, message, opts={}):
    response = proc_obj.response
    return response['msg'].append(message)


# Demo it
if __name__=='__main__':
    class Demo:
        def __init__(self):
            self.response = {'errs': [],
                             'msg' : []}
            pass
        pass

    import pprint
    demo = Demo()
    msg(demo, 'hi')
    pp = pprint.PrettyPrinter()
    pp.pprint(demo.response)



# -*- coding: utf-8 -*-
"""
    trepan.io
    ~~~~~~~~~

    Trepan Input and Output routines

    :copyright: Copyright 2013, 2018
    :license: GPL3, see LICENSE for details.
"""

__docformat__ = 'restructuredtext'

from trepan.misc import pyfiles
__all__ = pyfiles(__file__)



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013-2015 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import signal
import sys
import time

# Our local modules
from trepan.processor.command import base_cmd as Mbase_cmd
from trepan.lib import complete as Mcomplete

# This code is from StackOverflow:
#  http://stackoverflow.com/questions/35772001/how-to-handle-the-signal-in-python-on-windows-machine
if sys.platform != 'win32':
    kill = os.kill
    sleep = time.sleep
else:
    # adapt the conflated API on Windows.
    import threading

    sigmap = {signal.SIGINT: signal.CTRL_C_EVENT,
              signal.SIGBREAK: signal.CTRL_BREAK_EVENT}

    def kill(pid, signum):
        if signum in sigmap and pid == os.getpid():
            # we don't know if the current process is a
            # process group leader, so just broadcast
            # to all processes attached to this console.
            pid = 0
        thread = threading.current_thread()
        handler = signal.getsignal(signum)
        # work around the synchronization problem when calling
        # kill from the main thread.
        if (signum in sigmap and
            thread.name == 'MainThread' and
            callable(handler) and
            pid == 0):
            event = threading.Event()

            def handler_set_event(signum, frame):
                event.set()
                return handler(signum, frame)
            signal.signal(signum, handler_set_event)
            try:
                kill(pid, sigmap[signum])
                # busy wait because we can't block in the main
                # thread, else the signal handler can't execute.
                while not event.is_set():
                    pass
            finally:
                signal.signal(signum, handler)
        else:
            kill(pid, sigmap.get(signum, signum))

    if sys.version_info[0] > 2:
        sleep = time.sleep
    else:
        import errno

        # If the signal handler doesn't raise an exception,
        # time.sleep in Python 2 raises an EINTR IOError, but
        # Python 3 just resumes the sleep.

        def sleep(interval):
            '''sleep that ignores EINTR in 2.x on Windows'''
            while True:
                try:
                    t = time.time()
                    time.sleep(interval)
                except IOError as e:
                    if e.errno != errno.EINTR:
                        raise
                interval -= time.time() - t
                if interval <= 0:
                    break


class KillCommand(Mbase_cmd.DebuggerCommand):
    """**kill** [ *signal-number* ] [unconditional]

Send this process a POSIX signal ('9' for 'SIGKILL' or 'kill -SIGKILL')

9 is a non-maskable interrupt that terminates the program. If program
is threaded it may be expedient to use this command to terminate the program.

However other signals, such as those that allow for the debugged to
handle them can be sent.

Giving a negative number is the same as using its
positive value.

Examples:
--------

    kill                # non-interuptable, nonmaskable kill
    kill 9              # same as above
    kill -9             # same as above
    kill!               # same as above, but no confirmation
    kill unconditional  # same as above
    kill 15             # nicer, maskable TERM signal
    kill! 15            # same as above, but no confirmation

See also:
---------

`quit` for less a forceful termination command; `exit` for another way to force termination.

`run` and `restart` are ways to restart the debugged program.
"""

    aliases       = ('kill!',)
    category      = 'running'
    min_args      = 0
    max_args      = 1
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = False
    short_help    = 'Send this process a POSIX signal ("9" for "kill -9")'

    def complete(self, prefix):
        names = [sig for sig in signal.__dict__.keys() if
                 sig.startswith('SIG')]
        nums  = [str(eval("signal."+name)) for name in names]
        lnames = [sig.lower() for sig in names]
        completions = lnames + nums + ['unconditional']
        return Mcomplete.complete_token(completions, prefix.lower())

    def run(self, args):
        if sys.platform != 'win32':
            signo =  signal.SIGKILL
        else:
            signo =  signal.CTRL_BREAK_EVENT

        confirmed = False
        if len(args) <= 1:
            if '!' != args[0][-1]:
                confirmed = self.confirm('Really do a hard kill', False)
            else:
                confirmed = True
        elif 'unconditional'.startswith(args[1]):
            confirmed = True
        else:
            try:
                signo = abs(int(args[1]))
                confirmed = True
            except ValueError:
                pass
            pass

        if confirmed:
            import os
            # FIXME: check validity of signo.
            kill(os.getpid(), signo)
            pass
        return False  # Possibly not reached
    pass

if __name__ == '__main__':
    def handle(*args):
        print('signal received')
        pass
    signal.signal(28, handle)

    from trepan.processor.command import mock
    d, cp = mock.dbg_setup()
    command = KillCommand(cp)
    print(command.complete(''))
    command.run(['kill', 'wrong', 'number', 'of', 'args'])
    command.run(['kill', '28'])
    command.run(['kill!'])



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013, 2015, 2017, 2020 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os.path as osp

from trepan.processor.command.base_cmd import DebuggerCommand
from trepan.processor.cmdbreak import parse_break_cmd, set_break


class ContinueCommand(DebuggerCommand):
    """**continue** [*location*]

Leave the debugger read-eval print loop and continue
execution. Subsequent entry to the debugger however may occur via
breakpoints or explicit calls, or exceptions.

If *location* is given, a temporary breakpoint is set at that
position before continuing.

Examples:
---------

    continue          # Continue execution
    continue 5        # Continue with a one-time breakpoint at line 5
    continue basename # Go to os.path.basename if we have basename imported
    continue /usr/lib/python2.7/posixpath.py:110 # Possibly the same as
                                                 # the above using file
                                                 # and line number

See also:
---------

`step` `jump`, `next`, `finish` and `help syntax location`
"""

    category = "running"
    aliases = ("c",)
    execution_set = ["Running"]
    min_args = 0
    max_args = None
    name = osp.basename(__file__).split(".")[0]
    need_stack = True
    short_help = "Continue execution of debugged program"

    def run(self, args):
        if len(args) > 1:
            # FIXME: DRY this code. Better is to hook into tbreak.
            func, filename, lineno, condition = parse_break_cmd(
                self.proc, args
            )
            if not set_break(
                self, func, filename, lineno, condition, True, args
            ):
                return False
        self.core.step_events = None  # All events
        self.core.step_ignore = -1
        self.proc.continue_running = True  # Break out of command read loop
        return True

    pass


if __name__ == "__main__":
    import sys
    from trepan.debugger import Trepan

    d = Trepan()
    cmd = ContinueCommand(d.core.processor)
    cmd.proc.frame = sys._getframe()
    cmd.proc.setup()

    for c in (
        ["continue", "wrong", "number", "of", "args"],
        ["c", "5"],
        ["continue", "1+2"],
        ["c", "foo"],
    ):
        d.core.step_ignore = 0
        cmd.continue_running = False
        result = cmd.run(c)
        print("Run result: %s" % result)
        print(
            "step_ignore %d, continue_running: %s"
            % (d.core.step_ignore, cmd.continue_running,)
        )
        pass
    pass



# -*- coding: utf-8 -*-
# Copyright (C) 2013, 2015 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pyficache import highlight_string

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.lib import complete as Mcomplete


class InfoMacro(Mbase_subcmd.DebuggerSubcommand):
    """**info macro * ***

**info macro** *macro1* [*macro2* ..]

In the first form a list of the existing macro names are shown
in column format.

In the second form, all macro names and their definitions are shown.

In the last form the only definitions of the given macro names is shown."""

    min_abbrev = 1
    need_stack = True
    short_help = "List of defined macros"

    def complete(self, prefix):
        m = sorted(list(self.proc.macros.keys()) + ['*'])
        return Mcomplete.complete_token(m, prefix)

    def run(self, args):
        if len(args) > 0:
            if len(args) == 1 and '*' == args[0]:
                macro_names = list(self.proc.macros.keys())
            else:
                macro_names = args
                pass

            for macro_name in sorted(macro_names):
                if macro_name in self.proc.macros:
                    self.section("%s:" % macro_name)
                    string = self.proc.macros[macro_name][1]
                    highlight = self.settings['highlight']
                    if highlight in ['light', 'dark']:
                        string = highlight_string(string, highlight)
                        pass
                    self.msg("  %s" % string)
                else:
                    self.errmsg('%s is not a defined macro' % macro_name)
                    pass
                pass
        elif 0 == len(list(self.proc.macros.keys())):
            self.msg('No macros defined.')
        else:
            self.msg(self.columnize_commands(list(self.proc.macros.keys())))
            pass
        return
    pass

if __name__ == '__main__':
    # Demo it.
    from trepan.processor.command import mock, info as Minfo
    d, cp = mock.dbg_setup()
    i = Minfo.InfoCommand(cp)
    sub = InfoMacro(i)
    sub.run(["u"])
    print(sub.complete(''))
    pass



#!/usr/bin/env python3
'Unit test for the debugger lib breakpoint'
import re, unittest

from trepan.lib import breakpoint as Mbreakpoint


class TestBreakpoint(unittest.TestCase):

    def test_breakpoint(self):
        'Test breakpoint'
        bpmgr = Mbreakpoint.BreakpointManager()
        self.assertEqual(0, bpmgr.last())
        bp = bpmgr.add_breakpoint('foo', 5)
        self.assertFalse(not
                         re.search('1   breakpoint   keep yes   at .*foo:5',
                                   str(bp)))
        self.assertEqual('B', bp.icon_char())
        self.assertEqual(True, bp.enabled)
        bp.disable()
        self.assertFalse(not
                         re.search('1   breakpoint   keep no    at .*foo:5',
                         str(bp)))
        self.assertEqual(False, bp.enabled)
        self.assertEqual('b', bp.icon_char())
        self.assertEqual(1, bpmgr.last())
        self.assertEqual((False, 'Breakpoint number 10 out of range 1..1.'),
                         bpmgr.delete_breakpoint_by_number(10))
        self.assertEqual(['1'], bpmgr.bpnumbers(),
                         "Extracting disabled breakpoint-numbers")
        self.assertEqual((True, ''), bpmgr.delete_breakpoint_by_number(1))
        self.assertEqual((False, 'Breakpoint 1 previously deleted.'),
                          bpmgr.delete_breakpoint_by_number(1))
        bp2 = bpmgr.add_breakpoint('foo', 5, temporary=True)
        self.assertEqual('t', bp2.icon_char())
        self.assertEqual(['2'], bpmgr.bpnumbers(),
                         "Extracting breakpoint-numbers")

        count = 3
        for i in range(count):
            bp = bpmgr.add_breakpoint('bar', 6)
        filename = bp.filename
        self.assertEqual(count, len(bpmgr.delete_breakpoints_by_lineno(filename, 6)),
                         "delete_breakpoints_by_line when there are some")
        self.assertEqual(0, len(bpmgr.delete_breakpoints_by_lineno(filename, 6)),
                         "delete_breakpoints_by_line when there are none")
        self.assertNotEqual(0, len(bpmgr.bplist),
                            "There should still be some breakpoints before reset")
        bpmgr.reset()
        self.assertEqual(0, len(bpmgr.bplist),
                         "reset should remove all breakpoints")
        return

    def test_checkfuncname(self):
        'Test breakpoint.checkfuncname()'
        import inspect
        frame = inspect.currentframe()

        bpmgr = Mbreakpoint.BreakpointManager()
        bp    = bpmgr.add_breakpoint('foo', 5)

        self.assertEqual(False, Mbreakpoint.checkfuncname(bp, frame))

        def foo(bp, bpmgr):
            frame = inspect.currentframe()
            self.assertEqual(True, Mbreakpoint.checkfuncname(bp, frame))
            # frame.f_lineno is constantly updated. So adjust for line
            # the difference between the add_breakpoint and the check.
            bp3 = bpmgr.add_breakpoint('foo', frame.f_lineno+1)
            self.assertEqual(True, Mbreakpoint.checkfuncname(bp3, frame))
            self.assertEqual(False, Mbreakpoint.checkfuncname(bp3, frame))
            return

        bp2 = bpmgr.add_breakpoint(None, None, False, None, 'foo')
        foo(bp2, bpmgr)
        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013-2015 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os, sys

from trepan.processor.command import base_cmd as Mbase_cmd


class CDCommand(Mbase_cmd.DebuggerCommand):
    """**cd** *directory*

Set working directory to *directory* for debugger and program
being debugged. """

    aliases       = ('chdir',)
    category      = 'files'
    min_args      = 1
    max_args      = 1
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = False
    short_help    = ('Set working directory to DIR for debugger '
                     'and program being debugged')

    def run(self, args):
        try:
            os.chdir(args[1])
            self.msg('Working directory %s.' % os.getcwd())
        except OSError:
            self.errmsg('cd: %s.' % sys.exc_info()[1])
            pass
        return
    pass

if __name__ == '__main__':
    from trepan import debugger as Mdebugger
    d = Mdebugger.Trepan()
    cmd = CDCommand(d.core.processor)
    for c in (['cd', 'wrong', 'number', 'of', 'args'],
              ['cd', 'foo'],
              ['cd', '.'],
              ['cd', '/'],
              ):
        cmd.run(c)
        pass
    pass



#!/usr/bin/env python3
'Unit test for trepan.processor.command.alias and unalias'
import inspect, unittest

from trepan import debugger as Mdebugger


class TestAliasCommand(unittest.TestCase):
    '''Test 'alias' and 'unalias' commands'''

    def errmsg(self, msg):
        self.errors.append(msg)
        return

    def msg(self, msg):
        self.msgs.append(msg)
        return

    def setUp(self):
        self.errors = []
        self.msgs = []
        d                     = Mdebugger.Trepan()
        self.cmdproc          = d.core.processor
        self.cmdproc.curframe = inspect.currentframe()
        cmd                   = self.cmdproc.commands['alias']
        cmd.msg               = self.msg
        cmd.errmsg            = self.errmsg
        cmd                   = self.cmdproc.commands['unalias']
        cmd.msg               = self.msg
        cmd.errmsg            = self.errmsg
        return

    def check_alias(self, should_not_have, cmd_name, *args):
        self.cmdproc.msgs    = []
        self.cmdproc.errmsgs = []
        cmds    = self.cmdproc.commands
        arg_str = ' '.join(args)
        my_cmd  = cmds[cmd_name]
        newargs = [cmd_name]
        newargs += args
        my_cmd.run(newargs)
        if should_not_have:
            shoulda = ['', 'no ']
        else:
            shoulda = ['no ', '']
            pass
        self.assertEqual(should_not_have, len(self.msgs) == 0,
                         "Expecting %s%s for %s.\n Got %s" %
                         (shoulda[0], cmd_name, arg_str, self.msgs))
        self.assertEqual(not should_not_have, len(self.errors) == 0,
                         "Expecting %serror for #{arg_str}.\n Got #{errmsgs}" %
                         shoulda[1])
        return

    def is_alias_defined(self, alias_name):
        return alias_name in list(self.cmdproc.aliases.keys())

    def test_alias_unalias_command(self):
        self.assertEqual(False, len(self.cmdproc.aliases) == 0,
                         'There should be some aliases defined')

        self.assertEqual(False, self.is_alias_defined('ki'))
        # from trepan.api import debug
        # debug()
        self.check_alias(False, 'alias', 'ki', 'kill')
        self.assertEqual(True, self.is_alias_defined('ki'))
        self.check_alias(False, 'unalias', 'ki')
        self.assertEqual(False, self.is_alias_defined('ki'))
        return

    pass

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
'Unit test for trepan.processor.command.kill'
import sys, unittest

from trepan.processor.command import kill as Mkill

from cmdhelper import dbg_setup
import signal


class TestKill(unittest.TestCase):
    """Tests KillCommand class"""

    def setUp(self):
        self.signal_caught = False
        return

    def handle(self, *args):
        self.signal_caught = True
        return

    def test_kill(self):
        """Test processor.command.kill.KillCommand.run()"""
        signal.signal(28, self.handle)
        d, cp = dbg_setup()
        command = Mkill.KillCommand(cp)
        result = command.run(['kill', 'wrong', 'number', 'of', 'args'])
        self.assertFalse(result)
        self.assertFalse(self.signal_caught)
        if sys.platform != 'win32':
            result = command.run(['kill', '28'])
            self.assertFalse(result)
            self.assertTrue(self.signal_caught)
        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
'Unit test for trepan.bytecode'
import inspect, unittest

from trepan.lib import bytecode as Mcode
from xdis import IS_PYPY, PYTHON_VERSION


class TestByteCode(unittest.TestCase):

    def test_contains_make_function(self):
        def sqr(x):
            return x * x
        frame = inspect.currentframe()
        co = frame.f_code
        lineno = frame.f_lineno
        self.assertTrue(Mcode.stmt_contains_opcode(co, lineno-4,
                                                   'MAKE_FUNCTION'))
        self.assertFalse(Mcode.stmt_contains_opcode(co, lineno,
                                                    'MAKE_FUNCTION'))
        return

    def test_op_at_frame(self):
        frame = inspect.currentframe()
        if IS_PYPY or PYTHON_VERSION >= 3.7:
            call_opcode = 'CALL_METHOD'
        else:
            call_opcode = 'CALL_FUNCTION'

        self.assertEqual(call_opcode, Mcode.op_at_frame(frame))
        return

    def test_is_def_frame(self):
        # Not a "def" statement because frame is wrong spot
        frame = inspect.currentframe()
        self.assertFalse(Mcode.is_def_stmt('foo(): pass', frame))
        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2009, 2013, 2015 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
import inspect, os, re

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.lib import file
from trepan import clifns as Mclifns, misc as Mmisc


def find_function(funcname, filename):
    cre = re.compile(r'def\s+%s\s*[(]' % re.escape(funcname))
    try:
        fp = open(filename)
    except IOError:
        return None
    # consumer of this info expects the first line to be 1
    lineno = 1
    answer = None
    while True:
        line = fp.readline()
        if line == '':
            break
        if cre.match(line):
            answer = funcname, filename, lineno
            break
        lineno = lineno + 1
        pass
    fp.close()
    return answer

class InfoLine(Mbase_subcmd.DebuggerSubcommand):
    '''**info line**

Show information about the current line

See also:
---------
`info program`, `info frame`'''

    min_abbrev = 2
    max_args = 0
    need_stack = True
    short_help = 'Show current-line information'

    def lineinfo(self, identifier):
        failed = (None, None, None)
        # Input is identifier, may be in single quotes
        idstring = identifier.split("'")
        if len(idstring) == 1:
            # not in single quotes
            ident = idstring[0].strip()
        elif len(idstring) == 3:
            # quoted
            ident = idstring[1].strip()
        else:
            return failed
        if ident == '': return failed
        parts = ident.split('.')
        # Protection for derived debuggers
        if parts[0] == 'self':
            del parts[0]
            if len(parts) == 0:
                return failed
        # Best first guess at file to look at
        fname = self.proc.defaultFile()
        if len(parts) == 1:
            item = parts[0]
        else:
            # More than one part.
            # First is module, second is method/class
            m, f = file.lookupmodule('.'.join(parts[1:]),
                                     self.debugger.mainpyfile,
                                     self.core.canonic)
            if f:
                fname = f
            item = parts[-1]
        answer = find_function(item, fname)
        return answer or failed

    def run(self, args):
        """Current line number in source file"""
        # info line identifier
        if not self.proc.curframe:
            self.errmsg("No line number information available.")
            return
        if len(args) == 3:
            # lineinfo returns (item, file, lineno) or (None,)
            answer = self.lineinfo(args[2])
            if answer[0]:
                item, filename, lineno = answer
                if not os.path.isfile(filename):
                    filename = Mclifns.search_file(filename,
                                                   self.core.search_path,
                                                   self.main_dirname)
                self.msg('Line %s of "%s" <%s>' %
                         (lineno, filename, item))
            return
        filename=self.core.canonic_filename(self.proc.curframe)
        if not os.path.isfile(filename):
            filename = Mclifns.search_file(filename, self.core.search_path,
                                           self.main_dirname)
            pass

        filename = self.core.canonic_filename(self.proc.curframe)
        msg1 = 'Line %d of \"%s\"'  % (inspect.getlineno(self.proc.curframe),
                                       self.core.filename(filename))
        msg2 = ('at instruction %d' % self.proc.curframe.f_lasti)
        if self.proc.event:
            msg2 += ', %s event' % self.proc.event
            pass
        self.msg(Mmisc.wrapped_lines(msg1, msg2, self.settings['width']))
        return False
    pass

if __name__ == '__main__':
    from trepan.processor.command import mock, info as Minfo
    from trepan import debugger as Mdebugger
    d = Mdebugger.Trepan()
    d, cp = mock.dbg_setup(d)
    i = Minfo.InfoCommand(cp)
    sub = InfoLine(i)
    sub.run([])
    cp.curframe = inspect.currentframe()
    for width in (80, 200):
        sub.settings['width'] = width
        sub.run(['file.py', 'lines'])
        pass
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2009-2010, 2013, 2015 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

# Our local modules
from trepan.processor.command import base_cmd
from trepan.processor import cmdfns as Mcmdfns


class NextCommand(base_cmd.DebuggerCommand):

    aliases       = ('next+', 'next-', 'n', 'n-', 'n+')
    category      = 'running'
    execution_set = ['Running']
    min_args      = 0
    max_args      = 1
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = True
    short_help    = 'Step program without entering called functions'

    def run(self, args):
        """**next**[**+**|**-**] [*count*]

Step one statement ignoring steps into function calls at this level.

With an integer argument, perform `next` that many times. However if
an exception occurs at this level, or we *return*, *yield* or the
thread changes, we stop regardless of count.

A suffix of `+` on the command or an alias to the command forces to
move to another line, while a suffix of `-` does the opposite and
disables the requiring a move to a new line. If no suffix is given,
the debugger setting 'different-line' determines this behavior.

See also:
---------

`step`, `skip`, `jump` (there's no `hop` yet), `continue`, and
`finish` for other ways to progress execution.
"""

        if len(args) <= 1:
            step_ignore = 0
        else:
            step_ignore = self.proc.get_int(args[1], default=1,
                                            cmdname='next')
            if step_ignore is None: return False
            # 0 means stop now or step 1, so we subtract 1.
            step_ignore -= 1
            pass
        self.core.different_line   = \
            Mcmdfns.want_different_line(args[0],
                                        self.debugger.settings['different'])
        self.core.set_next(self.proc.frame, step_ignore)
        self.proc.continue_running = True   # Break out of command read loop
        return True
    pass

if __name__ == '__main__':
    from mock import MockDebugger
    d = MockDebugger()
    cmd = NextCommand(d.core.processor)
    for c in (['n', '5'],
              ['next', '1+2'],
              ['n', 'foo']):
        d.core.step_ignore = 0
        cmd.continue_running = False
        result = cmd.run(c)
        print('Run result: %s' % result)
        print('step_ignore %d, continue_running: %s' % (d.core.step_ignore,
                                                        cmd.continue_running,))
        pass
    for c in (['n'], ['next+'], ['n-']):
        d.core.step_ignore = 0
        cmd.continue_running = False
        result = cmd.run(c)
        print(cmd.core.different_line)
        pass
    pass



#  Copyright (c) 2017-2018 by Rocky Bernstein
from __future__ import print_function

from trepan.processor.parse.parser import (
    parse_bp_location, parse_range, parse_arange
    )
from trepan.processor.parse.parser import LocationError as PLocationError
from trepan.processor.parse.scanner import ScannerError
from spark_parser import GenericASTTraversal # , DEFAULT_DEBUG as PARSER_DEFAULT_DEBUG

from collections import namedtuple
Location = namedtuple("Location", "path line_number is_address method")
BPLocation = namedtuple("BPLocation", "location condition")
ListRange = namedtuple("ListRange", "first last")


class LocationError(Exception):
    def __init__(self, errmsg):
        self.errmsg = errmsg

    def __str__(self):
        return self.errmsg

class RangeError(Exception):
    def __init__(self, errmsg):
        self.errmsg = errmsg

    def __str__(self):
        return self.errmsg

class LocationGrok(GenericASTTraversal, object):

    def __init__(self, text):
        GenericASTTraversal.__init__(self, None)
        self.text = text
        self.result = None
        return

    def n_addr_location(self, node):
        # addr_location ::= location
        # addr_location ::= ADDRESS
        # addr_location ::= FUNCNAME (via location singleton reduce)
        # addr_location ::= NUMBER (via location singleton reduce)

        path, line_number, method = None, None, None
        n0 = node[0]
        if n0 in ('ADDRESS', 'FUNCNAME', 'NUMBER'):
            if n0 == 'ADDRESS':
                assert node[0].value[0] == '*'
                line_number = int(node[0].value[1:])
                self.result = Location(path, line_number, True, method)
            elif n0 == 'NUMBER':
                self.result = Location(path, node[0].value, True, method)
            else:
                self.result = Location(path, line_number, False, node[0].value[:-2])
            node.location = self.result
            self.prune()
        else:
            # print(node[0])
            assert node[0] == 'location'
            self.preorder(node[0])
            node.location = node[0].location


    def n_location(self, node):
        path, line_number, method = None, None, None
        if node[0] == 'FILENAME':
            path = node[0].value
            # If there is a line number, it is the last token of a location
            if len(node) > 1 and node[-1] == 'NUMBER':
                line_number = node[-1].value
        elif node[0] == 'FUNCNAME':
            method = node[0].value[:-2]
        elif node[0] == 'NUMBER':
            line_number = node[0].value
        else:
            assert True, "n_location: Something's is wrong; node[0] is %s" % node[0]
        self.result = Location(path, line_number, False, method)
        node.location = Location(path, line_number, False, method)
        self.prune()

    def n_NUMBER(self, node):
        self.result = Location(None, node.value, False, None)

    def n_FUNCNAME(self, node):
        self.result = Location(None, None, False, node.value[:-2])

    def n_location_if(self, node):
        location = None
        if node[0] == 'location':
            self.preorder(node[0])
            location = node[0].location

        if len(node) == 1:
            return
        if node[1] == 'IF':
            if_node = node[1]
        elif node[2] == 'IF':
            if_node = node[2]
        elif node[3] == 'IF':
            if_node = node[3]
        else:
            assert False, 'location_if: Something is wrong; cannot find "if"'

        condition = self.text[if_node.offset+len(if_node.value)+1:]

        # Pick out condition from string and location inside "IF" token
        self.result = BPLocation(location, condition)
        self.prune()

    # FIXME: DRY with range
    def n_arange(self, arange_node):
        if arange_node[0]  == 'range':
            arange_node = arange_node[0]
        l = len(arange_node)
        if 1 <= l <= 2:
            # arange ::= location
            # arange ::= DIRECTION
            # arange ::= FUNCNAME
            # arange ::= NUMBER
            # arange ::= OFFSET
            # arange ::= ADDRESS
            last_node = arange_node[-1]
            if last_node == 'location':
                self.preorder(arange_node[-1])
                self.result = ListRange(last_node.location, None)
            elif last_node == 'FUNCNAME':
                self.result = ListRange(Location(None, None, False, last_node.value[:-2]),
                                        None)
            elif last_node in ('NUMBER', 'OFFSET', 'ADDRESS'):
                if last_node == 'ADDRESS':
                    assert last_node.value[0] == '*'
                    is_address = True
                    value = int(last_node.value[1:])
                else:
                    is_address = False
                    value = last_node.value

                self.result = ListRange(Location(None, value, is_address, None),
                                        None)
            else:
                assert last_node == 'DIRECTION'
                self.result = ListRange(None, last_node.value)
                pass
            self.prune()
        elif l == 3:
            # arange ::= COMMA opt_space location
            # arange ::= location opt_space COMMA
            if arange_node[0] == 'COMMA':
                assert arange_node[-1] == 'location'
                self.preorder(arange_node[-1])
                self.result = ListRange(None, self.result)
                self.prune()
            else:
                assert arange_node[-1] == 'COMMA'
                assert arange_node[0] == 'location'
                self.preorder(arange_node[0])
                self.result = ListRange(arange_node[0].location, None)
                self.prune()
                pass
        elif l == 5:
            # arange ::= location opt_space COMMA opt_space {NUMBER | OFFSET | ADDRESS}
            assert arange_node[2] == 'COMMA'
            assert arange_node[-1] in ('NUMBER', 'OFFSET', 'ADDRESS')
            self.preorder(arange_node[0])
            self.result = ListRange(arange_node[0].location, arange_node[-1].value)
            self.prune()
        else:
            raise RangeError("Something is wrong")
        return

    def n_range(self, range_node):
        l = len(range_node)
        if 1 <= l <= 2:
            # range ::= location
            # range ::= DIRECTION
            # range ::= FUNCNAME
            # range ::= NUMBER
            # range ::= OFFSET
            last_node = range_node[-1]
            if last_node == 'location':
                self.preorder(range_node[-1])
                self.result = ListRange(last_node.location, None)
            elif last_node == 'FUNCNAME':
                self.result = ListRange(Location(None, None, False, last_node.value[:-2]),
                                        None)
            elif last_node in ('NUMBER', 'OFFSET'):
                self.result = ListRange(Location(None, last_node.value, False, None),
                                        None)
            else:
                assert last_node == 'DIRECTION'
                self.result = ListRange(None, last_node.value)
                pass
            self.prune()
        elif l == 3:
            # range ::= COMMA opt_space location
            # range ::= location opt_space COMMA
            if range_node[0] == 'COMMA':
                assert range_node[-1] == 'location'
                self.preorder(range_node[-1])
                self.result = ListRange(None, self.result)
                self.prune()
            else:
                assert range_node[-1] == 'COMMA'
                assert range_node[0] == 'location'
                self.preorder(range_node[0])
                self.result = ListRange(range_node[0].location, None)
                self.prune()
                pass
        elif l == 5:
            # range ::= location opt_space COMMA opt_space {NUMBER|OFFSET}
            assert range_node[2] == 'COMMA'
            assert range_node[-1] in ('NUMBER', 'OFFSET')
            self.preorder(range_node[0])
            self.result = ListRange(range_node[0].location, range_node[-1].value)
            self.prune()
        else:
            raise RangeError("Something is wrong")
        return

    def default(self, node):
        if node not in frozenset(("""opt_space tokens token bp_start range_start arange_start
                                  IF FILENAME COLON COMMA SPACE DIRECTION""".split())):
            assert False, ("Something's wrong: you missed a rule for %s" % node.kind)

    def traverse(self, node):
        return self.preorder(node)


def build_bp_expr(string, show_tokens=False, show_ast=False, show_grammar=False):
    parser_debug = {'rules': False, 'transition': False,
                    'reduce': show_grammar,
                    'errorstack': None, 'dups': False
                    # 'context': True, 'dups': True
                        }
    parsed = parse_bp_location(string, show_tokens=show_tokens,
                               parser_debug=parser_debug)
    assert parsed == 'bp_start'
    if show_ast:
        print(parsed)
    walker = LocationGrok(string)
    walker.traverse(parsed)
    bp_expr = walker.result
    if isinstance(bp_expr, Location):
        bp_expr = BPLocation(bp_expr, None)
    location = bp_expr.location
    assert location.line_number is not None or location.method
    return bp_expr

def build_range(string, show_tokens=False, show_ast=False, show_grammar=False):
    parser_debug = {'rules': False, 'transition': False,
                    'reduce': show_grammar,
                    'errorstack': None,
                    'context': False, 'dups': True
                        }
    parsed = parse_range(string, show_tokens=show_tokens,
                               parser_debug=parser_debug)
    if show_ast:
        print(parsed)
    assert parsed == 'range_start'
    walker = LocationGrok(string)
    walker.traverse(parsed)
    list_range = walker.result
    return list_range

# FIXME: DRY with build_range
def build_arange(string, show_tokens=False, show_ast=False, show_grammar=False):
    parser_debug = {'rules': False, 'transition': False,
                    'reduce': show_grammar,
                    'errorstack': None,
                    'context': True, 'dups': True
                        }
    parsed = parse_arange(string, show_tokens=show_tokens,
                          parser_debug=parser_debug)
    if show_ast:
        print(parsed)
    assert parsed == 'arange_start'
    walker = LocationGrok(string)
    walker.traverse(parsed)
    list_range = walker.result
    return list_range

if __name__ == '__main__':
    # FIXME: make sure the below is in a test
    def doit(fn, line):
        print("=" * 30)
        print(line)
        print("+" * 30)
        try:
            result = fn(line)
            print(result)
        except ScannerError as e:
            print("Scanner error")
            print(e.text)
            print(e.text_cursor)
        except PLocationError as e:
            print("Parser error at or near")
            print(e.text)
            print(e.text_cursor)

    # lines = """
    # /tmp/foo.py:12
    # /tmp/foo.py line 12
    # 12
    # ../foo.py:5
    # gcd()
    # foo.py line 5 if x > 1
    # """.splitlines()
    # for line in lines:
    #     if not line.strip():
    #         continue
    #     print("=" * 30)
    #     print(line)
    #     print("+" * 30)
    #     bp_expr = build_bp_expr(line)
    #     print(bp_expr)
    # lines = (
    #     "/tmp/foo.py:12 , 5",
    #     "-"
    #     "+"
    #     "../foo.py:5",
    #     "../foo.py:5 ",
    #     ", 5",
    #     "6 , +2",
    #     ", /foo.py:5",
    #     )
    lines = (
        "*10",
        "*10",
        "/tmp/foo.py:1, *5",
        "../foo.py:0,  +5",
        "../foo.py:5,  *30",
        "*0, *10",
        ", /foo.py:5",
        )
    for line in lines:
        line = line.strip()
        if not line:
            continue
        doit(build_arange, line)



#!/usr/bin/env python3
'Unit test for trepan.processor.command.step'
import inspect, os, sys, unittest


class TestFinishCommand(unittest.TestCase):
    """Tests FinishCommand class"""

    def test_finish(self):
        """Test processor.command.finish.FinishCommand.run()"""
        from cmdhelper import dbg_setup
        d, cp = dbg_setup()
        import trepan.processor.command.finish as Mfinish
        command = Mfinish.FinishCommand(cp)
        for c in ((['finish', '5'], True),
                  (['finish', '0*5+1'], True)):
            command.continue_running = False
            command.proc.stack = [(sys._getframe(0), 14)]
            result = command.run(c[0])
            self.assertEqual(c[1], result)
            pass
        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2008-2009, 2013, 2015, 2017 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan import misc as Mmisc

class InfoProgram(Mbase_subcmd.DebuggerSubcommand):
    """**info program**

Execution status of the program. Listed are:

* Program name

* Instruction PC

* Reason the program is stopped.

See also:
---------

`info line`, `info args`, `info frame`, `info pc`"""

    min_abbrev = 2  # Need at least info pr
    max_args   = 0
    need_stack = True
    short_help = 'Execution status of the program'

    def run(self, args):
        """Execution status of the program."""
        mainfile = self.core.filename(None)
        if self.core.is_running():
            if mainfile:
                part1 = "Python program '%s' is stopped" % mainfile
            else:
                part1 = 'Program is stopped'
                pass
            if self.proc.event:
                msg = 'via a %s event.' % self.proc.event
            else:
                msg = '.'
            self.msg(Mmisc.wrapped_lines(part1, msg,
                                         self.settings['width']))
            if self.proc.curframe:
                self.msg("PC offset is %d." % self.proc.curframe.f_lasti)

            if self.proc.event == 'return':
                val = self.proc.event_arg
                part1 = 'Return value is'
                self.msg(Mmisc.wrapped_lines(part1, self.proc._saferepr(val),
                                             self.settings['width']))
                pass
            elif self.proc.event == 'exception':
                exc_type, exc_value, exc_tb = self.proc.event_arg
                self.msg('Exception type: %s' %
                         self.proc._saferepr(exc_type))
                if exc_value:
                    self.msg('Exception value: %s' %
                             self.proc._saferepr(exc_value))
                    pass
                pass
            self.msg('It stopped %s.' % self.core.stop_reason)
            if self.proc.event in ['signal', 'exception', 'c_exception']:
                self.msg('Note: we are stopped *after* running the '
                         'line shown.')
                pass
        else:
            if mainfile:
                part1 = "Python program '%s'" % mainfile
                msg   = "is not currently running. "
                self.msg(Mmisc.wrapped_lines(part1, msg,
                                             self.settings['width']))
            else:
                self.msg('No Python program is currently running.')
                pass
            self.msg(self.core.execution_status)
            pass
        return False
    pass

if __name__ == '__main__':
    from trepan.processor.command import mock, info as Minfo
    d, cp = mock.dbg_setup()
    i = Minfo.InfoCommand(cp)
    sub = InfoProgram(i)
    sub.run([])



# This file is needs to be multi-lingual in both Python and POSIX
# shell which "exec()" or "source" it respectively.

# This file should define a variable VERSION which we use as the
# debugger version number.

VERSION="1.0.1"  # noqa



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013-2015 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inspect, os

# Our local modules
from trepan.processor.command import base_cmd as Mbase_cmd


class EditCommand(Mbase_cmd.DebuggerCommand):
    """**edit** *position*

Edit specified file or module.
With no argument, edits file containing most recent line listed.

See also:
---------

`list`
"""

    aliases       = ('ed',)
    category      = 'files'
    min_args      = 0
    max_args      = 1
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = False
    short_help    = 'Edit specified file or module'

    def run(self, args):
        curframe = self.proc.curframe
        if len(args) == 1:
            if curframe is None:
                self.errmsg('edit: no stack to pick up position from. '
                            'Use edit FILE:LINE form.')
                return
            filename = curframe.f_code.co_filename
            lineno   = curframe.f_lineno
        elif len(args) == 2:
            (modfunc, filename, lineno) = self.proc.parse_position(args[1])
            if inspect.ismodule(modfunc) and lineno is None and len(args) > 2:
                val = self.proc.get_an_int(args[1],
                                           'Line number expected, got %s.' %
                                           args[1])
                if val is None: return
                lineno = val
                pass
            elif lineno is None:
                self.errmsg('edit: no linenumber provided')
                return
            pass
        editor = 'ex'
        if 'EDITOR' in os.environ:
            editor = os.environ['EDITOR']
            pass
        if os.path.exists(filename):
            os.system("%s +%d %s" % (editor, lineno, filename))
        else:
            self.errmsg("edit: file %s doesn't exist" % filename)
            pass
        return
    pass

if __name__ == '__main__':
    from trepan import debugger as Mdebugger
    d = Mdebugger.Trepan()
    cmd = EditCommand(d.core.processor)
    for c in (['edit'],
              ['edit', './edit.py:34'],
              ['edit', './noogood.py'],
              ):
        cmd.run(c)
        pass
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2012-2013, 2015 Rocky Bernstein
#
# Our local modules
from trepan.processor.command import base_subcmd as Mbase_subcmd
from trepan.processor import cmdfns as Mcmdfns


class SetWidth(Mbase_subcmd.DebuggerSubcommand):
    """**set width** *number*

Set the number of characters the debugger thinks are in a line.

See also:
--------

`show width`
"""

    in_list    = True
    min_abbrev = len('wid')
    short_help = 'Set the width of the terminal'

    def run(self, args):
        Mcmdfns.run_set_int(self, ' '.join(args),
                            "The 'width' command requires a line width",
                            0, None)
        return
    pass

if __name__ == '__main__':
    from trepan.processor.command.set_subcmd import __demo_helper__ as Mhelper
    sub = Mhelper.demo_run(SetWidth)
    d = sub.proc.debugger
    sub.run(['100'])
    print(d.settings['width'])
    pass



# -*- coding: utf-8 -*-
#   Copyright (C) 2009, 2013-2015, 2017 Rocky Bernstein <rocky@gnu.org>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Module for Server (i.e. program to communication-device) interaction"""
import atexit

# Our local modules
from trepan import interface as Minterface
from trepan.inout import tcpserver as Mtcpserver, fifoserver as Mfifoserver
from trepan.interfaces import comcodes as Mcomcodes

DEFAULT_INIT_CONNECTION_OPTS = {'IO': 'TCP',
                                'PORT': 1955}

class ServerInterface(Minterface.TrepanInterface):
    """Interface for debugging a program but having user control
    reside outside of the debugged process, possibly on another
    computer."""

    def __init__(self, inout=None, out=None, connection_opts={}):
        atexit.register(self.finalize)

        opts = DEFAULT_INIT_CONNECTION_OPTS.copy()
        opts.update(connection_opts)
        self.inout = None  # initialize in case assignment below fails
        if inout:
            self.inout = inout
        else:
            self.server_type = opts['IO']
            if 'FIFO' == self.server_type:
                self.inout = Mfifoserver.FIFOServer()
            else:
                self.inout = Mtcpserver.TCPServer(opts=opts)
                pass
            pass
        # For Compatability
        self.output = self.inout
        self.input  = self.inout
        self.interactive = True  # Or at least so we think initially
        self.histfile = None
        return

    def close(self):
        """ Closes both input and output """
        if self.inout:
            self.inout.close()
        return

    def confirm(self, prompt, default):
        """ Called when a dangerous action is about to be done to make sure
        it's okay. `prompt' is printed; user response is returned."""
        while True:
            try:
                self.write_confirm(prompt, default)
                reply = self.readline('').strip().lower()
            except EOFError:
                return default
            if reply in ('y', 'yes'):
                return True
            elif reply in ('n', 'no'):
                return False
            else:
                self.msg("Please answer y or n.")
                pass
            pass
        return default

    def errmsg(self, str, prefix="** "):
        """Common routine for reporting debugger error messages.
           """
        return self.msg("%s%s" %(prefix, str))

    def finalize(self, last_wishes=Mcomcodes.QUIT):
        # print exit annotation
        if self.is_connected():
            self.inout.writeline(last_wishes)
            pass
        self.close()
        return

    def is_connected(self):
        """ Return True if we are connected """
        return self.inout and 'connected' == self.inout.state

    def msg(self, msg):
        """ used to write to a debugger that is connected to this
        server; `str' written will have a newline added to it
        """
        self.inout.writeline(Mcomcodes.PRINT + msg)
        return

    def msg_nocr(self, msg):
        """ used to write to a debugger that is connected to this
        server; `str' written will not have a newline added to it
        """
        self.inout.write(Mcomcodes.PRINT +  msg)
        return

    def read_command(self, prompt):
        return self.readline(prompt)

    def read_data(self):
        return self.inout.read_data()

    def readline(self, prompt, add_to_history=True):
        if prompt:
            self.write_prompt(prompt)
            pass
        coded_line = self.inout.read_msg()
        self.read_ctrl = coded_line[0]
        return coded_line[1:]

    def state(self):
        """ Return connected """
        return self.inout.state

    def write_prompt(self, prompt):
        return self.inout.writeline(Mcomcodes.PROMPT + prompt)

    def write_confirm(self, prompt, default):
        if default:
            code = Mcomcodes.CONFIRM_TRUE
        else:
            code = Mcomcodes.CONFIRM_FALSE
            pass
        return self.inout.writeline(code + prompt)

    pass

# Demo
if __name__=='__main__':
    connection_opts={'IO': 'TCP', 'PORT': 1954}
    intf = ServerInterface(connection_opts=connection_opts)
    pass



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013, 2015, 2018-2019 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
from getopt import getopt, GetoptError

# Our local modules
from trepan.processor.command import base_cmd as Mbase_cmd
from trepan.lib import stack as Mstack


class BacktraceCommand(Mbase_cmd.DebuggerCommand):
    """**backtrace** [*options*] [*count*]

Print backtrace of all stack frames, or innermost *count* frames.

With a negative argument, print outermost -*count* frames.

An arrow indicates the 'current frame'. The current frame determines
the context used for many debugger commands such as expression
evaluation or source-line listing.

*options* are:

   -d | --deparse - show deparsed call position
   -s | --source  - show source code line
   -f | --full    - locals of each frame
   -h | --help    - give this help

Examples:
---------

   backtrace      # Print a full stack trace
   backtrace 2    # Print only the top two entries
   backtrace -1   # Print a stack trace except the initial (least recent) call.
   backtrace -s   # show source lines in listing
   backtrace -d   # show deparsed source lines in listing
   backtrace -f   # show with locals
   backtrace -df  # show with deparsed calls and locals
   backtrace --deparse --full   # same as above

See also:
---------

`frame`, `locals`, `global`, `deparse`, `list`.
"""

    aliases       = ('bt', 'where')
    category      = 'stack'
    min_args      = 0
    max_args      = 4
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = True
    short_help   = 'Print backtrace of stack frames'

    def run(self, args):

        try:
            opts, args = getopt(args[1:], "hfds",
                                "help deparse full source".split())
        except GetoptError as err:
            # print help information and exit:
            print(str(err))  # will print something like "option -a not recognized"
            return

        bt_opts = {'width': self.settings['width']}
        for o, a in opts:
            if o in ("-h", "--help"):
                self.proc.commands['help'].run(['help', 'backtrace'])
                return
            elif o in ("-d", "--deparse"):
                bt_opts['deparse'] = True
            elif o in ("-f", "--full"):
                bt_opts['full'] = True
            elif o in ("-s", "--source"):
                bt_opts['source'] = True
            else:
                self.errmsg("unhandled option '%s'" % o)
            pass


        if len(args) > 0:
            at_most = len(self.proc.stack)
            if at_most == 0:
                self.errmsg("Stack is empty.")
                return False
            min_value = - at_most + 1
            count = self.proc.get_int(args[0], min_value = min_value,
                                      cmdname = 'backtrace',
                                      default=0, at_most = at_most)
            if count is None: return False
            if count < 0:
                count =  at_most - count
                pass
            elif 0 == count: count = None
        else:
            count = None
            pass

        if not self.proc.curframe:
            self.errmsg("No stack.")
            return False
        Mstack.print_stack_trace(self.proc, count,
                                 color=self.settings['highlight'],
                                 opts=bt_opts)
        return False

    pass

if __name__ == '__main__':
    from trepan.processor import cmdproc
    from trepan import debugger
    d            = debugger.Debugger()
    cp           = d.core.processor
    command      = BacktraceCommand(cp)
    command.run(['backtrace', 'wrong', 'number', 'of', 'args'])

    def nest_me(cp, command, i):
        import inspect
        if i > 1:
            cp.curframe = inspect.currentframe()
            cp.stack, cp.curindex = cmdproc.get_stack(cp.curframe, None, None,
                                                      cp)
            print('-' * 10)
            command.run(['backtrace'])
            print('-' * 10)
            command.run(['backtrace', '1'])
        else:
            nest_me(cp, command, i+1)
        return

    def ignore_me(cp, command, i):
        print('=' * 10)
        nest_me(cp, command, 1)
        print('=' * 10)
        cp.core.add_ignore(ignore_me)
        nest_me(cp, command, 1)
        return
    cp.forget()
    command.run(['backtrace'])
    print('-' * 10)
    ignore_me(cp, command, 1)
    command.run(['backtrace', '1'])
    print('-' * 10)
    command.run(['backtrace', '-1'])
    print('-' * 10)
    command.run(['backtrace', '3'])
    print('-' * 10)
    command.run(['backtrace', '-2'])
    print('-' * 10)
    pass



#!/usr/bin/env python
'Unit test for trepan.processor.command.pr'
import unittest

from trepan import debugger
from trepan.processor.command.p import PCommand


class TestP(unittest.TestCase):
    """Tests PrintCommand class"""

    def setUp(self):
        self.errors = []
        self.msgs = []
        return

    def errmsg(self, msg):
        self.errors.append(msg)
        return

    def msg(self, msg):
        self.msgs.append(msg)
        return

    def test_p(self):
        import inspect
        d           = debugger.Trepan()
        cp          = d.core.processor
        cp.curframe = inspect.currentframe()
        cmd         = PCommand(cp)
        cmd.msg     = self.msg
        cmd.errmsg  = self.errmsg
        me = 10  # NOQA
        cmd.run([cmd.name, 'me'])
        self.assertEqual('10', self.msgs[-1])
        cmd.run([cmd.name, '/x', 'me'])
        self.assertEqual("'0xa'", self.msgs[-1])
        cmd.run([cmd.name, '/o', 'me'])
        self.assertEqual("'0o12'", self.msgs[-1])
        return

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"General integration tests"
import unittest

import helper as Mhelper
from xdis import PYTHON_VERSION


class GeneralTests(unittest.TestCase):

    def test_step(self):
        """Test stepping, set skip, set trace"""
        if PYTHON_VERSION >= 3.8:
            right_template = '%s-38.right'
        else:
            right_template = None
        result=Mhelper.run_debugger(testname='step',
                                    dbgr_opts='--basename --highlight=plain',
                                    python_file='gcd.py',
                                    right_template = right_template)
        self.assertEqual(True, result, "debugger 'step' command comparision")
        return
    pass

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
'Unit test for trepan.processor.command.break'

import platform, os, unittest

from trepan import debugger
from trepan.processor import cmdbreak as Mcmdbreak
Mbreak = __import__('trepan.processor.command.break', None, None, ['*'])


class TestBreakCommand(unittest.TestCase):

    def setUp(self):
        self.errors = []
        self.msgs = []
        return

    def errmsg(self, msg):
        self.errors.append(msg)
        return

    def msg(self, msg):
        self.msgs.append(msg)
        return

    def parse_break_cmd(self, proc, cmd):
        proc.current_command = cmd
        args = cmd.split(' ')
        return Mcmdbreak.parse_break_cmd(proc, args)

    def test_parse_break_cmd(self):
        import inspect
        d               = debugger.Trepan()
        cp              = d.core.processor
        cp.curframe     = inspect.currentframe()
        self.cmd        = Mbreak.BreakCommand(cp)
        self.cmd.msg    = self.msg
        self.cmd.errmsg = self.errmsg
        proc = self.cmd.proc

        fn, fi, li, cond = self.parse_break_cmd(proc, 'break')
        self.assertEqual((None, True, True),
                         (fn, fi.endswith('test-break.py'), li > 1))

        fn, fi, li, cond = self.parse_break_cmd(proc, 'break 11')
        self.assertEqual((None, True, 11),
                         (fn, fi.endswith('test-break.py'), li))

        if 'APPVEYOR' not in os.environ:
            if platform.system() == 'Windows':
                brk_cmd = 'b """%s""":8' % __file__
            else:
                brk_cmd = 'b %s:8' % __file__

                fn, fi, li, cond = self.parse_break_cmd(proc, brk_cmd)
                self.assertEqual((None, True, 8),
                                 (fn, isinstance(fi, str), li))

        def foo():
            return 'bar'

        fn, fi, li, cond = self.parse_break_cmd(proc, 'break foo()')
        self.assertEqual((foo, True, True),
                         (fn, fi.endswith('test-break.py'), li > 1))

        fn, fi, li, cond = self.parse_break_cmd(proc, 'break food()')
        self.assertEqual((None, None, None, None), (fn, fi, li, cond))

        fn, fi, li, cond = self.parse_break_cmd(proc, 'b os.path:5')
        self.assertEqual((os.path, True, 5),
                         (fn, isinstance(fi, str), li))

        fn, fi, li, cond = self.parse_break_cmd(proc, 'b os.path.join()')
        self.assertEqual((os.path.join, True, True),
                         (fn, isinstance(fi, str), li > 1))

        fn, fi, li, cond = self.parse_break_cmd(proc, 'break if True')
        self.assertEqual((None, True, True),
                         (fn, fi.endswith('test-break.py'), li > 1))

        fn, fi, li, cond = self.parse_break_cmd(proc, 'b foo() if True')
        self.assertEqual((foo, True, True),
                         (fn, fi.endswith('test-break.py'), li > 1))

        fn, fi, li, cond = self.parse_break_cmd(proc, 'br os.path:10 if True')
        self.assertEqual((True, 10),
                         (isinstance(fi, str), li))

        # FIXME:
        # Try:
        #  a breakpoint with a symlink in the filename.
        #  breakpoint with a single quotes and embedded black
        #  breakpoint with a double quotes and embedded \,
        #  Triple quote things
        #
        # Also, add a unit test for canonic.

        return

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#   Copyright (C) 2015 Rocky Bernstein <rocky@gnu.org>
''' Location routines'''

import pyficache, linecache, tempfile
from trepan.lib import stack as Mstack


def format_location(proc_obj):
    """Show where we are. GUI's and front-end interfaces often
    use this to update displays. So it is helpful to make sure
    we give at least some place that's located in a file.
    """
    i_stack = proc_obj.curindex
    if i_stack is None or proc_obj.stack is None:
        return False
    location = {}
    core_obj = proc_obj.core
    dbgr_obj = proc_obj.debugger

    # Evaluation routines like "exec" don't show useful location
    # info. In these cases, we will use the position before that in
    # the stack.  Hence the looping below which in practices loops
    # once and sometimes twice.
    while i_stack >= 0:
        frame_lineno = proc_obj.stack[i_stack]
        i_stack -= 1
        frame, lineno = frame_lineno

        filename = Mstack.frame2file(core_obj, frame)

        location['filename'] = filename
        location['fn_name']  = frame.f_code.co_name
        location['lineno']   = lineno

        if '<string>' == filename and dbgr_obj.eval_string:
            filename = pyficache.unmap_file(filename)
            if '<string>' == filename:
                fd = tempfile.NamedTemporaryFile(suffix='.py',
                                                 prefix='eval_string',
                                                 delete=False)
                fd.write(bytes(dbgr_obj.eval_string, 'UTF-8'))
                fd.close()
                pyficache.remap_file(fd.name, '<string>')
                filename = fd.name
                pass
            pass

        opts = {
            'reload_on_change' : proc_obj.settings('reload'),
            'output'           : 'plain'
            }
        line = pyficache.getline(filename, lineno, opts)
        if not line:
            line = linecache.getline(filename, lineno,
                                     proc_obj.curframe.f_globals)
            pass

        if line and len(line.strip()) != 0:
            location['text'] = line
            pass
        if '<string>' != filename: break
        pass

    return location

def print_location(proc_obj, event=None):
    response = proc_obj.response
    response['name'] = 'status'
    response['location'] = format_location(proc_obj)
    if event:
        response['event'] = event
        if event in ['return', 'exception']:
            val = proc_obj._saferepr(proc_obj.event_arg)
            event['arg'] = val
            pass
        pass
    proc_obj.intf[-1].msg(response)
    return


# Demo it
if __name__=='__main__':
    class MockDebugger:
        def __init__(self):
            self.eval_string = None
        pass

    class MockProcessor:
        def __init__(self, core_obj):
            self.curindex = 0
            self.stack = []
            self.core = core_obj
            self.debugger = MockDebugger()
            self.opts = {'highlight': 'plain', 'reload': False}
            pass

        def settings(self, key):
            return self.opts[key]
        pass

    class MockCore:
        def filename(self, fn): return fn

        def canonic_filename(self, frame): return frame.f_code.co_filename
        pass

    core = MockCore()
    cmdproc = MockProcessor(core)

    import sys
    cmdproc.curframe = cmdproc.frame = sys._getframe()
    cmdproc.stack.append((sys._getframe(), 10))

    import pprint
    pp = pprint.PrettyPrinter()
    pp.pprint(format_location(cmdproc))

    def test(cmdproc, pp):
        cmdproc.stack[0:0] = [(sys._getframe(1), 1)]
        pp.pprint(format_location(cmdproc))
        pass
    eval('test(cmdproc, pp)')
    cmdproc.debugger.eval_string = 'Fooled you!'
    eval('test(cmdproc, pp)')
    pass



# -*- coding: utf-8 -*-
#
#  Copyright (C) 2009-2010, 2013, 2015-2018 Rocky Bernstein
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import inspect
import pyficache
from trepan import misc as Mmisc
from trepan.processor.parse.semantics import build_bp_expr
from trepan.processor.parse.parser import LocationError
from trepan.processor.parse.scanner import ScannerError
from trepan.processor.location import resolve_location

def set_break(cmd_obj, func, filename, lineno, condition, temporary,
              args, force=False):
    if lineno is None:
        part1 = ("I don't understand '%s' as a line number, function name,"
                 % ' '.join(args[1:]))
        msg = Mmisc.wrapped_lines(part1, "or file/module plus line number.",
                                  cmd_obj.settings['width'])
        cmd_obj.errmsg(msg)
        return False
    if filename is None:
        filename = cmd_obj.proc.curframe.f_code.co_filename
        filename = cmd_obj.core.canonic(filename)
        pass
    if func is None:
        ok_linenos = pyficache.trace_line_numbers(filename)
        if not ok_linenos or lineno not in ok_linenos:
            part1 = ('File %s' % cmd_obj.core.filename(filename))
            msg = Mmisc.wrapped_lines(part1,
                                      "is not stoppable at line %d." %
                                      lineno, cmd_obj.settings['width'])
            cmd_obj.errmsg(msg)
            if force:
                cmd_obj.msg("Breakpoint set although it may never be reached")
            else:
                return False
        pass
    bp =  cmd_obj.core.bpmgr.add_breakpoint(filename, lineno, temporary,
                                         condition, func)
    if func and inspect.isfunction(func):
        cmd_obj.msg('Breakpoint %d set on calling function %s()'
                 % (bp.number, func.__name__))
        part1 = 'Currently this is line %d of file'  % lineno
        msg = Mmisc.wrapped_lines(part1, cmd_obj.core.filename(filename),
                                  cmd_obj.settings['width'])
    else:
        part1 = ( 'Breakpoint %d set at line %d of file'
                  % (bp.number, lineno))
        msg = Mmisc.wrapped_lines(part1, cmd_obj.core.filename(filename),
                                  cmd_obj.settings['width'])
        pass
    cmd_obj.msg(msg)
    return True

INVALID_PARSE_BREAK = (None, None, None, None)
def parse_break_cmd(proc, args):
    if proc.current_command is None:
        proc.errmsg("Internal error")
        return INVALID_PARSE_BREAK

    text = proc.current_command[len(args[0])+1:]
    if len(args) > 1 and args[1] == 'if':
        location = '.'
        condition = text[text.find('if ')+3:]
    elif text == '':
        location = '.'
        condition  = None
    else:
        try:
            bp_expr   = build_bp_expr(text)
        except LocationError as e:
            proc.errmsg("Error in parsing breakpoint expression at or around:")
            proc.errmsg(e.text)
            proc.errmsg(e.text_cursor)
            return INVALID_PARSE_BREAK
        except ScannerError as e:
            proc.errmsg("Lexical error in parsing breakpoint expression at or around:")
            proc.errmsg(e.text)
            proc.errmsg(e.text_cursor)
            return INVALID_PARSE_BREAK

        location  = bp_expr.location
        condition = bp_expr.condition

    location = resolve_location(proc, location)
    if location:
        return location.method, location.path, location.line_number, condition
    else:
        return INVALID_PARSE_BREAK


# Demo it
if __name__=='__main__':
    from trepan.processor.command import mock as Mmock
    from trepan.processor.cmdproc import CommandProcessor
    import sys
    d = Mmock.MockDebugger()
    cmdproc = CommandProcessor(d.core)
    # print '-' * 10
    # print_source_line(sys.stdout.write, 100, 'source_line_test.py')
    # print '-' * 10
    cmdproc.frame = sys._getframe()
    cmdproc.setup()
    for cmd in (
            # "break '''c:\\tmp\\foo.bat''':1",
            # 'break """/Users/My Documents/foo.py""":2',
            # "break",
            # "break 10",
            # "break if True",
            # "break cmdproc.py:5",
            # "break set_break()",
            "break 4 if i==5",
            # "break cmdproc.setup()",
            ):
        args = cmd.split(' ')
        cmdproc.current_command = cmd
        print(parse_break_cmd(cmdproc, args))
    pass



#!/usr/bin/env python3
'Unit test for debugger info file'
import inspect, unittest

from trepan import debugger as Mdebugger
from trepan.processor.command import info as Minfo
from trepan.processor.command.info_subcmd import files as MinfoFile

from cmdhelper import dbg_setup


class TestInfoFile(unittest.TestCase):

    # FIXME: put in a more common place
    # Possibly fix up Mock to include this
    def setup_io(self, command):
        self.clear_output()
        command.msg = self.msg
        command.errmsg = self.errmsg
        command.msg_nocr = self.msg_nocr
        return

    def clear_output(self):
        self.msgs = []
        self.errmsgs = []
        self.last_was_newline = True
        return

    def msg_nocr(self, msg):
        if len(self.msgs) > 0:
            self.msgs[-1] += msg
        else:
            self.msgs += msg
            pass
        return

    def msg(self, msg):
        self.msgs += [msg]
        return

    def errmsg(self, msg):
        self.errmsgs.append(msg)
        pass

    def test_info_file(self):
        d = Mdebugger.Trepan()
        d, cp = dbg_setup(d)
        command = Minfo.InfoCommand(cp, 'info')

        sub = MinfoFile.InfoFiles(command)
        self.setup_io(sub)
        sub.run([])
        self.assertEqual([], self.msgs)
        cp.curframe = inspect.currentframe()
        for width in (80, 200):
            # sub.settings['width'] = width
            sub.run(['test-info-file.py', 'lines'])
            sub.run([])
            pass
        pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#  Copyright (C) 2009, 2013, 2015, 2017 Rocky Bernstein
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

# Our local modules
from trepan.processor.command import base_cmd as Mbase_cmd
from trepan.processor import complete as Mcomplete


class DeleteCommand(Mbase_cmd.DebuggerCommand):
    """**delete** [*bpnumber* [*bpnumber*...]]

Delete some breakpoints.

Arguments are breakpoint numbers with spaces in between.  To delete
all breakpoints, give no argument.  Without
arguments, clear all breaks (but first ask for confirmation).

See also:
---------
`clear`
"""
    category      = 'breakpoints'
    aliases       = ('delete!',)
    min_args      = 0
    max_args      = None
    name          = os.path.basename(__file__).split('.')[0]
    need_stack    = False
    short_help    = 'Delete some breakpoints or auto-display expressions'

    complete = Mcomplete.complete_bpnumber

    def run(self, args):
        if len(args) <= 1:
            if '!' != args[0][-1]:
                confirmed = self.confirm('Delete all breakpoints', False)
            else:
                confirmed = True

            if confirmed:
                self.msg(self.core.bpmgr.delete_all_breakpoints())
            return

        for arg in args[1:]:
            i = self.proc.get_int(arg, min_value=1, default=None,
                                  cmdname='delete')
            if i is None: continue

            success, msg = self.core.bpmgr.delete_breakpoint_by_number(i)
            if not success:
                self.errmsg(msg)
            else:
                self.msg('Deleted breakpoint %d' % i)
                pass
            pass
        return


if __name__ == '__main__':
    from trepan import debugger as Mdebugger
    d = Mdebugger.Trepan()
    command = DeleteCommand(d.core.processor)
    pass
